@@ -70,7 +70,7 @@ backlog = 2048
 #
 
 workers = 1
-worker_class = 'egg:gunicorn#sync'
+worker_class = 'sync'
 worker_connections = 1000
 timeout = 30
 keepalive = 2
@@ -200,3 +200,23 @@ def pre_exec(server):
 
 def when_ready(server):
     server.log.info("Server is ready. Spwawning workers")
+
+def worker_int(worker):
+    worker.log.info("worker received INT or TERM signal")
+
+    ## get traceback info
+    import threading, sys, traceback
+    id2name = dict([(th.ident, th.name) for th in threading.enumerate()])
+    code = []
+    for threadId, stack in sys._current_frames().items():
+        code.append("\n# Thread: %s(%d)" % (id2name.get(threadId,""),
+            threadId))
+        for filename, lineno, name, line in traceback.extract_stack(stack):
+            code.append('File: "%s", line %d, in %s' % (filename,
+                lineno, name))
+            if line:
+                code.append("  %s" % (line.strip()))
+    worker.log.debug("\n".join(code))
+
+def worker_abort(worker):
+    worker.log.info("worker received SIGABRT signal")
