@@ -5,6 +5,8 @@
 #
 # Example code from Eventlet sources
 
+import os
+import pprint
 from wsgiref.validate import validator
 import sys
 
@@ -12,6 +14,10 @@ from gunicorn import __version__
 #@validator
 def app(environ, start_response):
     """Simplest possible application object"""
+
+    errors = environ['wsgi.errors']
+#    pprint.pprint(('ENVIRON', environ), stream=errors)
+
     data = b'Hello, World!\n'
     status = '200 OK'
 
@@ -22,5 +28,4 @@ def app(environ, start_response):
         ("Test", "test тест"),
     ]
     start_response(status, response_headers)
-    print("test")
     return iter([data])
