@@ -3,8 +3,6 @@
 # This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
-from __future__ import with_statement
-
 import errno
 import os
 import random
@@ -93,35 +91,39 @@ class Arbiter(object):
         if 'GUNICORN_FD' in os.environ:
             self.log.reopen_files()
 
+        self.worker_class = self.cfg.worker_class
         self.address = self.cfg.address
         self.num_workers = self.cfg.workers
-        self.debug = self.cfg.debug
         self.timeout = self.cfg.timeout
         self.proc_name = self.cfg.proc_name
-        self.worker_class = self.cfg.worker_class
 
-        if self.cfg.debug:
-            self.log.debug("Current configuration:")
-            for config, value in sorted(self.cfg.settings.items(),
-                    key=lambda setting: setting[1]):
-                self.log.debug("  %s: %s", config, value.value)
+        self.log.debug('Current configuration:\n{0}'.format(
+            '\n'.join(
+                '  {0}: {1}'.format(config, value.value)
+                for config, value
+                in sorted(self.cfg.settings.items(),
+                          key=lambda setting: setting[1]))))
+
+        # set enviroment' variables
+        if self.cfg.env:
+            for k, v in self.cfg.env.items():
+                os.environ[k] = v
 
         if self.cfg.preload_app:
-            if not self.cfg.debug:
-                self.app.wsgi()
-            else:
-                self.log.warning("debug mode: app isn't preloaded.")
+            self.app.wsgi()
 
     def start(self):
         """\
         Initialize the arbiter. Start listening and set pidfile if needed.
         """
         self.log.info("Starting gunicorn %s", __version__)
+
         self.pid = os.getpid()
         if self.cfg.pidfile is not None:
             self.pidfile = Pidfile(self.cfg.pidfile)
             self.pidfile.create(self.pid)
         self.cfg.on_starting(self)
+
         self.init_signals()
         if not self.LISTENERS:
             self.LISTENERS = create_sockets(self.cfg, self.log)
@@ -129,8 +131,7 @@ class Arbiter(object):
         listeners_str = ",".join([str(l) for l in self.LISTENERS])
         self.log.debug("Arbiter booted")
         self.log.info("Listening at: %s (%s)", listeners_str, self.pid)
-        self.log.info("Using worker: %s",
-                self.cfg.settings['worker_class'].get())
+        self.log.info("Using worker: %s", self.cfg.worker_class_str)
 
         self.cfg.when_ready(self)
 
@@ -168,7 +169,6 @@ class Arbiter(object):
         self.manage_workers()
         while True:
             try:
-                self.reap_workers()
                 sig = self.SIG_QUEUE.pop(0) if len(self.SIG_QUEUE) else None
                 if sig is None:
                     self.sleep()
@@ -206,6 +206,7 @@ class Arbiter(object):
 
     def handle_chld(self, sig, frame):
         "SIGCHLD handling"
+        self.reap_workers()
         self.wakeup()
 
     def handle_hup(self):
@@ -218,8 +219,8 @@ class Arbiter(object):
         self.log.info("Hang up: %s", self.master_name)
         self.reload()
 
-    def handle_quit(self):
-        "SIGQUIT handling"
+    def handle_term(self):
+        "SIGTERM handling"
         raise StopIteration
 
     def handle_int(self):
@@ -227,7 +228,7 @@ class Arbiter(object):
         self.stop(False)
         raise StopIteration
 
-    def handle_term(self):
+    def handle_quit(self):
         "SIGTERM handling"
         self.stop(False)
         raise StopIteration
@@ -274,7 +275,7 @@ class Arbiter(object):
             self.num_workers = 0
             self.kill_workers(signal.SIGQUIT)
         else:
-            self.log.info("SIGWINCH ignored. Not daemonized")
+            self.log.debug("SIGWINCH ignored. Not daemonized")
 
     def wakeup(self):
         """\
@@ -294,6 +295,7 @@ class Arbiter(object):
             self.log.info("Reason: %s", reason)
         if self.pidfile is not None:
             self.pidfile.unlink()
+        self.cfg.on_exit(self)
         sys.exit(exit_status)
 
     def sleep(self):
@@ -301,8 +303,16 @@ class Arbiter(object):
         Sleep until PIPE is readable or we timeout.
         A readable PIPE means a signal occurred.
         """
+        if self.WORKERS:
+            worker_values = list(self.WORKERS.values())
+            oldest = min(w.tmp.last_update() for w in worker_values)
+            timeout = self.timeout - (time.time() - oldest)
+            # The timeout can be reached, so don't wait for a negative value
+            timeout = max(timeout, 1.0)
+        else:
+            timeout = 1.0
         try:
-            ready = select.select([self.PIPE[0]], [], [], 1.0)
+            ready = select.select([self.PIPE[0]], [], [], timeout)
             if not ready[0]:
                 return
             while os.read(self.PIPE[0], 1):
@@ -324,14 +334,13 @@ class Arbiter(object):
         killed gracefully  (ie. trying to wait for the current connection)
         """
         self.LISTENERS = []
-        sig = signal.SIGQUIT
+        sig = signal.SIGTERM
         if not graceful:
-            sig = signal.SIGTERM
+            sig = signal.SIGQUIT
         limit = time.time() + self.cfg.graceful_timeout
         while self.WORKERS and time.time() < limit:
             self.kill_workers(sig)
             time.sleep(0.1)
-            self.reap_workers()
         self.kill_workers(signal.SIGKILL)
 
     def reexec(self):
@@ -346,17 +355,32 @@ class Arbiter(object):
             self.master_name = "Old Master"
             return
 
+        environ = self.cfg.env_orig.copy()
         fds = [l.fileno() for l in self.LISTENERS]
-        os.environ['GUNICORN_FD'] = ",".join([str(fd) for fd in fds])
+        environ['GUNICORN_FD'] = ",".join([str(fd) for fd in fds])
 
         os.chdir(self.START_CTX['cwd'])
         self.cfg.pre_exec(self)
 
-        os.execvpe(self.START_CTX[0], self.START_CTX['args'], os.environ)
+        # exec the process using the original environnement
+        os.execvpe(self.START_CTX[0], self.START_CTX['args'], environ)
 
     def reload(self):
         old_address = self.cfg.address
 
+        # reset old environement
+        for k in self.cfg.env:
+            if k in self.cfg.env_orig:
+                # reset the key to the value it had before
+                # we launched gunicorn
+                os.environ[k] = self.cfg.env_orig[k]
+            else:
+                # delete the value set by gunicorn
+                try:
+                    del os.environ[k]
+                except KeyError:
+                    pass
+
         # reload conf
         self.app.reload()
         self.setup(self.app)
@@ -367,7 +391,7 @@ class Arbiter(object):
         # do we need to change listener ?
         if old_address != self.cfg.address:
             # close all listeners
-            [l.close for l in self.LISTENERS]
+            [l.close() for l in self.LISTENERS]
             # init new listeners
             self.LISTENERS = create_sockets(self.cfg, self.log)
             self.log.info("Listening at: %s", ",".join(str(self.LISTENERS)))
@@ -400,15 +424,20 @@ class Arbiter(object):
         """
         if not self.timeout:
             return
-        for (pid, worker) in self.WORKERS.items():
+        workers = list(self.WORKERS.items())
+        for (pid, worker) in workers:
             try:
                 if time.time() - worker.tmp.last_update() <= self.timeout:
                     continue
             except ValueError:
                 continue
 
-            self.log.critical("WORKER TIMEOUT (pid:%s)", pid)
-            self.kill_worker(pid, signal.SIGKILL)
+            if not worker.aborted:
+                self.log.critical("WORKER TIMEOUT (pid:%s)", pid)
+                worker.aborted = True
+                self.kill_worker(pid, signal.SIGABRT)
+            else:
+                self.kill_worker(pid, signal.SIGKILL)
 
     def reap_workers(self):
         """\
@@ -436,8 +465,8 @@ class Arbiter(object):
                         continue
                     worker.tmp.close()
         except OSError as e:
-            if e.errno == errno.ECHILD:
-                pass
+            if e.errno != errno.ECHILD:
+                raise
 
     def manage_workers(self):
         """\
@@ -512,7 +541,8 @@ class Arbiter(object):
         Kill all workers with the signal `sig`
         :attr sig: `signal.SIG*` value
         """
-        for pid in self.WORKERS.keys():
+        worker_pids = list(self.WORKERS.keys())
+        for pid in worker_pids:
             self.kill_worker(pid, sig)
 
     def kill_worker(self, pid, sig):
