@@ -3,25 +3,24 @@
 # This file is part of the pywebmachine package released
 # under the MIT license.
 
-from __future__ import with_statement
-
 import t
 
 import inspect
 import os
 import random
-import urlparse
 
 from gunicorn.config import Config
 from gunicorn.http.errors import ParseException
 from gunicorn.http.parser import RequestParser
+from gunicorn.six import urlparse, execfile_
+from gunicorn import six
 
 dirname = os.path.dirname(__file__)
 random.seed()
 
 def uri(data):
     ret = {"raw": data}
-    parts = urlparse.urlparse(data)
+    parts = urlparse(data)
     ret["scheme"] = parts.scheme or ''
     ret["host"] = parts.netloc.rsplit(":", 1)[0] or None
     ret["port"] = parts.port or 80
@@ -41,8 +40,9 @@ def uri(data):
 def load_py(fname):
     config = globals().copy()
     config["uri"] = uri
-    execfile(fname, config)
-    return config["request"]
+    config["cfg"] = Config()
+    execfile_(fname, config)
+    return config
 
 class request(object):
     def __init__(self, fname, expect):
@@ -53,10 +53,10 @@ class request(object):
         if not isinstance(self.expect, list):
             self.expect = [self.expect]
 
-        with open(self.fname) as handle:
+        with open(self.fname, 'rb') as handle:
             self.data = handle.read()
-        self.data = self.data.replace("\n", "").replace("\\r\\n", "\r\n")
-        self.data = self.data.replace("\\0", "\000")
+        self.data = self.data.replace(b"\n", b"").replace(b"\\r\\n", b"\r\n")
+        self.data = self.data.replace(b"\\0", b"\000")
 
     # Functions for sending data to the parser.
     # These functions mock out reading from a
@@ -68,26 +68,45 @@ class request(object):
 
     def send_lines(self):
         lines = self.data
-        pos = lines.find("\r\n")
+        pos = lines.find(b"\r\n")
         while pos > 0:
             yield lines[:pos+2]
             lines = lines[pos+2:]
-            pos = lines.find("\r\n")
+            pos = lines.find(b"\r\n")
         if len(lines):
             yield lines
 
     def send_bytes(self):
-        for d in self.data:
-            yield d
+        for d in str(self.data.decode("latin1")):
+            yield bytes(d.encode("latin1"))
 
     def send_random(self):
-        maxs = len(self.data) / 10
+        maxs = round(len(self.data) / 10)
         read = 0
         while read < len(self.data):
             chunk = random.randint(1, maxs)
             yield self.data[read:read+chunk]
             read += chunk
 
+    def send_special_chunks(self):
+        """Meant to test the request line length check.
+
+        Sends the request data in two chunks, one having a
+        length of 1 byte, which ensures that no CRLF is included,
+        and a second chunk containing the rest of the request data.
+
+        If the request line length check is not done properly,
+        testing the ``tests/requests/valid/099.http`` request
+        fails with a ``LimitRequestLine`` exception.
+
+        """
+        chunk = self.data[:1]
+        read = 0
+        while read < len(self.data):
+            yield self.data[read:read+len(chunk)]
+            read += len(chunk)
+            chunk = self.data[read:]
+
     # These functions define the sizes that the
     # read functions will read with.
 
@@ -142,7 +161,7 @@ class request(object):
         while len(body):
             if body[:len(data)] != data:
                 raise AssertionError("Invalid data read: %r" % data)
-            if '\n' in data[:-1]:
+            if b'\n' in data[:-1]:
                 raise AssertionError("Embedded new line: %r" % data)
             body = body[len(data):]
             data = self.szread(req.body.readline, sizes)
@@ -164,7 +183,7 @@ class request(object):
         """
         data = req.body.readlines()
         for line in data:
-            if '\n' in line[:-1]:
+            if b'\n' in line[:-1]:
                 raise AssertionError("Embedded new line: %r" % line)
             if line != body[:len(line)]:
                 raise AssertionError("Invalid body data read: %r != %r" % (
@@ -181,7 +200,7 @@ class request(object):
         This skips sizes because there's its not part of the iter api.
         """
         for line in req.body:
-            if '\n' in line[:-1]:
+            if b'\n' in line[:-1]:
                 raise AssertionError("Embedded new line: %r" % line)
             if line != body[:len(line)]:
                 raise AssertionError("Invalid body data read: %r != %r" % (
@@ -190,7 +209,7 @@ class request(object):
         if len(body):
             raise AssertionError("Failed to read entire body: %r" % body)
         try:
-            data = iter(req.body).next()
+            data = six.next(iter(req.body))
             raise AssertionError("Read data after body finished: %r" % data)
         except StopIteration:
             pass
@@ -198,7 +217,7 @@ class request(object):
     # Construct a series of test cases from the permutations of
     # send, size, and match functions.
 
-    def gen_cases(self):
+    def gen_cases(self, cfg):
         def get_funs(p):
             return [v for k, v in inspect.getmembers(self) if k.startswith(p)]
         senders = get_funs("send_")
@@ -213,19 +232,25 @@ class request(object):
 
         ret = []
         for (mt, sz, sn) in cfgs:
-            mtn = mt.func_name[6:]
-            szn = sz.func_name[5:]
-            snn = sn.func_name[5:]
+            if hasattr(mt, 'funcname'):
+                mtn = mt.func_name[6:]
+                szn = sz.func_name[5:]
+                snn = sn.func_name[5:]
+            else:
+                mtn = mt.__name__[6:]
+                szn = sz.__name__[5:]
+                snn = sn.__name__[5:]
+
             def test_req(sn, sz, mt):
-                self.check(sn, sz, mt)
+                self.check(cfg, sn, sz, mt)
             desc = "%s: MT: %s SZ: %s SN: %s" % (self.name, mtn, szn, snn)
             test_req.description = desc
             ret.append((test_req, sn, sz, mt))
         return ret
 
-    def check(self, sender, sizer, matcher):
+    def check(self, cfg, sender, sizer, matcher):
         cases = self.expect[:]
-        p = RequestParser(Config(), sender())
+        p = RequestParser(cfg, sender())
         for req in p:
             self.same(req, sizer, matcher, cases.pop(0))
         t.eq(len(cases), 0)
@@ -242,37 +267,24 @@ class request(object):
         t.eq(req.trailers, exp.get("trailers", []))
 
 class badrequest(object):
-    def __init__(self, fname, expect):
+    def __init__(self, fname):
         self.fname = fname
         self.name = os.path.basename(fname)
 
-        self.expect = expect
-        if not isinstance(self.expect, list):
-            self.expect = [self.expect]
-
         with open(self.fname) as handle:
             self.data = handle.read()
         self.data = self.data.replace("\n", "").replace("\\r\\n", "\r\n")
         self.data = self.data.replace("\\0", "\000")
+        self.data = self.data.encode('latin1')
 
     def send(self):
-        maxs = len(self.data) / 10
+        maxs = round(len(self.data) / 10)
         read = 0
         while read < len(self.data):
             chunk = random.randint(1, maxs)
             yield self.data[read:read+chunk]
             read += chunk
 
-    def check(self):
-        cases = self.expect[:]
-        p = RequestParser(Config(), self.send())
-        try:
-            [req for req in p]
-        except Exception, inst:
-            exp = cases.pop(0)
-            if not issubclass(exp, Exception):
-                raise TypeError("Test case is not an exception calss: %s" % exp)
-            t.istype(inst, exp)
-            return
-
-
+    def check(self, cfg):
+        p = RequestParser(cfg, self.send())
+        six.next(p)
